"use strict";
exports.__esModule = true;
var ts = require("typescript");
var TestTransformer_1 = require("./TestTransformer");
/**
 * Transforms test files in the describe/it format, delegating test transformation to TestTransformer
 */
var DescribeItFileTransformer = (function () {
    function DescribeItFileTransformer(context, program, options) {
        this.context = context;
        this.program = program;
        this.options = options || {};
        this.testTransformer = new TestTransformer_1.TestTransformer(context, program);
    }
    DescribeItFileTransformer.prototype.visitSourceFile = function (node) {
        var _this = this;
        // Skip filtered files
        if (this.options.fileFilter && !this.options.fileFilter(node.fileName))
            return node;
        return ts.visitEachChild(node, function (child) { return _this.visitBodyChild(child); }, this.context);
    };
    DescribeItFileTransformer.prototype.visitBodyChild = function (node) {
        if (!ts.isExpressionStatement(node))
            return node;
        if (!ts.isCallExpression(node.expression))
            return node;
        return ts.updateStatement(node, this.visitBodyChildExpression(node.expression));
    };
    DescribeItFileTransformer.prototype.visitBodyChildExpression = function (node) {
        if (!ts.isIdentifier(node.expression))
            return node;
        if (node.arguments.length !== 2)
            return node;
        var args = node.arguments.slice();
        var func = args[1];
        if (!(ts.isArrowFunction(func) || ts.isFunctionExpression(func)))
            return node;
        switch (node.expression.getText()) {
            case "describe":
                args[1] = this.visitDescribeFunction(func);
                break;
            case "it":
                args[1] = this.visitItFunction(func);
                break;
        }
        return ts.updateCall(node, node.expression, node.typeArguments, args);
    };
    DescribeItFileTransformer.prototype.visitDescribeFunction = function (node) {
        if (!ts.isBlock(node.body))
            return node;
        if (ts.isFunctionExpression(node)) {
            return ts.updateFunctionExpression(node, node.modifiers, node.asteriskToken, node.name, node.typeParameters, node.parameters, node.type, this.visitDescribeBody(node.body));
        }
        else {
            if (!ts.isBlock(node.body))
                return node;
            return ts.updateArrowFunction(node, node.modifiers, node.typeParameters, node.parameters, node.type, this.visitDescribeBody(node.body));
        }
    };
    DescribeItFileTransformer.prototype.visitDescribeBody = function (node) {
        var _this = this;
        return ts.visitEachChild(node, function (child) { return _this.visitBodyChild(child); }, this.context);
    };
    DescribeItFileTransformer.prototype.visitItFunction = function (node) {
        if (!ts.isBlock(node.body))
            return node;
        if (ts.isExpressionStatement(node.body))
            return node;
        if (ts.isFunctionExpression(node)) {
            return ts.updateFunctionExpression(node, node.modifiers, node.asteriskToken, node.name, node.typeParameters, node.parameters, node.type, this.testTransformer.visitTestBody(node.body));
        }
        else {
            return ts.updateArrowFunction(node, node.modifiers, node.typeParameters, node.parameters, node.type, this.testTransformer.visitTestBody(node.body));
        }
    };
    return DescribeItFileTransformer;
}());
exports.DescribeItFileTransformer = DescribeItFileTransformer;
