"use strict";
exports.__esModule = true;
var ts = require("typescript");
/**
 * Transforms test bodies, independent of testing framework / file structure
 */
var TestTransformer = (function () {
    function TestTransformer(context, program, options) {
        this.program = program;
        this.context = context;
    }
    TestTransformer.prototype.visitTestBody = function (node) {
        var diagnostics = this.program.getSemanticDiagnostics(node.getSourceFile());
        var containedDiagnostics = diagnostics.filter(function (diagnostic) {
            // I'm not sure when this location info might be undefined, just ignoring for now
            if (diagnostic.start === undefined)
                return false;
            if (diagnostic.length === undefined)
                return false;
            if (diagnostic.start < node.getStart())
                return false;
            if (diagnostic.start + diagnostic.length > node.getEnd())
                return false;
            return true;
        });
        var throwStatements = containedDiagnostics.map(function (diagnostic) {
            var throwStatement = ts.createThrow(ts.createNew(ts.createIdentifier("Error"), [], [ts.createLiteral(diagnostic.messageText.toString())]));
            ts.setSourceMapRange(throwStatement, {
                pos: diagnostic.start,
                end: diagnostic.start + diagnostic.length
            });
            return throwStatement;
        });
        return ts.createBlock(throwStatements, true);
    };
    return TestTransformer;
}());
exports.TestTransformer = TestTransformer;
