#!/usr/bin/env node
"use strict";
exports.__esModule = true;
var ts = require("typescript");
var path = require("path");
var minimatch = require("minimatch");
var commandpost = require("commandpost");
var _1 = require("./");
function build(glob, project) {
    var projectPath = project || "./";
    var configPath = ts.findConfigFile(projectPath, ts.sys.fileExists);
    var basePath = path.resolve(path.dirname(configPath)); /*?*/
    var configReadResult = ts.readConfigFile(configPath, ts.sys.readFile);
    if (configReadResult.error)
        throw new Error("Error reading tsconfig.json");
    var config = ts.parseJsonConfigFileContent(configReadResult.config, ts.sys, basePath);
    var program = ts.createProgram(config.fileNames, config.options);
    var transformer = _1.makeTransformer(program);
    program.getSourceFiles()
        .filter(function (file) { return minimatch(file.fileName, glob); })
        .forEach(function (file) { return program.emit(file, undefined, undefined, undefined, {
        before: [transformer]
    }); });
}
var post = commandpost
    .create("tsst <glob>")
    .option("-p, --project <path>", "Specify a project directory")
    .action(function (opts, args) {
    build(args.glob, opts.project[0]);
});
commandpost
    .exec(post, process.argv)["catch"](function (err) {
    if (err instanceof Error) {
        console.log(err.message);
    }
    else {
        console.log(err);
    }
    process.exit(1);
});
